﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            clcInfo_Monk.lua
--
--    Copyright (C) 2012 Geminior
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  To a great extend this is based on the clcInfo_Rogue by Deronda.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Geminior (@ Earthen Ring)
------------------------------------------------------------------------------
-- Description:
--   Global namespace for clcInfo_Monk.  The purpose is to avoid repetition.
--   However, since this module generates icons and bars that update many times
--   per second, locals should be used whenever possible and table lookups
--   avoided to speed things up.
--------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if clcInfo_Monk then return end     -- don't load if already loaded
local ENGLISH_CLASS_NAME = "MONK"
local _, englishClass  = UnitClass("player") -- only load for monks
if englishClass ~= ENGLISH_CLASS_NAME then return end

-----------------NAMESPACE---------------------------------------
clcInfo_Monk = {}
clcInfo_Monk.VERSION = 9
clcInfo_Monk.NOTIFY_VERSION = 1

clcInfo_Monk.CLASS = ENGLISH_CLASS_NAME

-----------------MONK SPELLS------------------------------------
clcInfo_Monk.spells = {
    --Brewmaster
	["Keg Smash"] = 121253,
	["Guard"] = 115295,
	["Light Stagger"] = 124275,
	["Moderate Stagger"] = 124274,
	["Heavy Stagger"] = 124273,
	["Shuffle"] = 115307,
	["Elusive Brew"] = 115308,
	["Brewing: Elusive Brew"] = 128938,
	["Purifying Brew"] = 119582,
	["Breath of Fire"] = 115181,
    --Windwalker
	["Rising Sun Kick"] = 107428,
	["Fists of Fury"] = 113656,
	["Energizing Brew"] = 115288,
	["Tigereye Brew"] = 116740,
	["Combo Breaker: Blackout Kick"] = 116768,
	["Combo Breaker: Tiger Palm"] = 118864,
	["Spinning Fire Blossom"]  = 115073,
	["Flying Serpent Kick"]  = 101545,
	["Touch of Karma"]  = 122470,
    --generic monk
	["Tiger Palm"] = 100787,
	["Blackout Kick"] = 100784,
	["Jab"] = 100780,
	["Touch of Death"] = 115080,
	["Expel Harm"] = 115072,
	["Fortifying Brew"] = 115203,
	["Provoke"]  = 115546,
	["Tiger Power"] = 125359,
	["Death Note"]  = 121125,
	["Invoke Xuen, the White Tiger"]  = 123904,
	["Crackling Jade Lightning"]  = 117952,
	["Diffuse Magic"]  = 122783,
	["Zen Meditation"]  = 115176,
	["Power Strikes"] = 129914,
}

-- container for modules registered
clcInfo_Monk.modules = {}

local L = LibStub("AceLocale-3.0"):GetLocale("clcInfo_Monk")

function clcInfo_Monk:SetSpellInfo(spellKey, spellId)
	local spellName = GetSpellInfo(spellId)
    if not spellName then spellName = L["Unknown Spell"] end
    self.spells[spellKey] = { id = spellId, name = spellName }
end

--localize spells
for key, value in pairs(clcInfo_Monk.spells) do
	clcInfo_Monk:SetSpellInfo(key, value)
end

--Generic Monk cds for all specs
clcInfo_Monk.basic_cds = {
    ["Fortifying Brew"]	= clcInfo_Monk.spells["Fortifying Brew"].id,
	["Diffuse Magic"]	= clcInfo_Monk.spells["Diffuse Magic"].id,
	["Zen Meditation"]	= clcInfo_Monk.spells["Zen Meditation"].id,
	["Invoke Xuen, the White Tiger"]  = clcInfo_Monk.spells["Invoke Xuen, the White Tiger"].id,
}

-- Constants
local gcdSpellId = clcInfo_Monk.spells["Jab"].id
local PowerStrikes = clcInfo_Monk.spells["Power Strikes"].name

---FUNCTIONS: Frequently called------------------------------------------------
-- Ability Icon Functions -> keep these fast!
local GetTime = GetTime
local GetSpellCooldown = GetSpellCooldown
local GetPowerRegen = GetPowerRegen
local UnitPower = UnitPower
local IsSpellKnown = IsSpellKnown
local UnitAura = UnitAura
local UnitSpellHaste = UnitSpellHaste
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local min, max = min, max
local ipairs = ipairs
local clcInfo_debug = clcInfo.debug
local emod = clcInfo.env
local IconSpell = clcInfo.env.IconSpell

-- locals for keeping track of time and resources
local playerLevel, playerMaxLevel
local current_time
local gcd_duration = 0
local activeChannelEnds = 0
local activeResources

--current ability id, next ability id
local cur_id, next_id

-----------------FUNCTIONS---------------------------------------
-- Configuration functions - performance not an issue

-- Print a message to the chat frame. outlevel can be 'info', 'warn' or 'err', 'info' being the default
function clcInfo_Monk.OutputMessage(msg, outlevel)
	local c = "ff41a25d"
	if outlevel == "warn" then
		c = "ffdee029"
	elseif outlevel == "err" then
		c = "ffea4a12"
	end
	
	msg = string.format("|cffffe566<clcInfo_Monk>|r|c%s %s|r", c, msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

-- Removes all occurences from a numeric table returning a new table with the remaining values in the same relational order. Original table is left unchanged
function clcInfo_Monk.StripFromArray(array, valueToRemove)
	local tmp = {}
	local newIdx = 1
	for i,v in ipairs(array) do
		if v ~= valueToRemove then
			tmp[newIdx] = v
			newIdx = newIdx + 1
		end
	end

	return tmp
end

-- Create the class module table with the name in class_module_name
-- Return the module and the module name.
function clcInfo_Monk:RegisterModule(moduleDefaults)
    local class_module_name = "____monk_" .. string.lower(moduleDefaults.moduleName)
    local class_module = clcInfo:RegisterClassModule(class_module_name)
	
	-- set the values as specified by the module
	class_module.specIdx = moduleDefaults.specIdx
	class_module.cmdPrioArg = moduleDefaults.cmdPrioArg
	class_module.moduleName = class_module_name

	table.insert(self.modules, class_module)
	
    return class_module
end

function clcInfo_Monk.GetSpecGroup(class_module)
	-- Must be done this way since the GetSpecialization will return nil when wow is loaded. It only returns the correct number after a reloadui which is odd
	-- find out if this is the primary or secondary spec
	if class_module.specIdx == GetSpecialization(nil, nil, 1) then
		return 1
	elseif class_module.specIdx == GetSpecialization(nil, nil, 2) then
		return 2
	end

	return nil
end

-- given a buff or debuff, the unit, and a filter, returns the time left based
-- on the current time and time left on the gcd
-- returns 0 if no such buff or debuff is present
function clcInfo_Monk.GetTimeLeft(unit, spell, filter)
    local time_left = 0
    local name, _, _, count, _, _, expires = UnitAura(unit, spell, nil, filter)
    if name then
        time_left = expires - current_time
    end
    return time_left, count
end
local GetTimeLeft = clcInfo_Monk.GetTimeLeft

-- given a spell id returns the cd left based on the current time variables
function clcInfo_Monk.GetCooldown(id, adjust)
	adjust = adjust or 0
    local start, duration = GetSpellCooldown(id)
    local cd = start + duration - current_time + adjust
    if cd < 0 then return 0 end
    return cd
end

local function InitResources()
	-- resource store, encapsulating the various monk resources used by abilities
	local res =
	{	
		chi = 0,
		last_chi = 0,
		chiMax = 4,
		energy = 100,
		energy_regen = 10,
		energy_regen_base = 10,
		energy_regen_boost = 0,
		energy_regen_boost_end = 0,
		spell_haste = 1,
		poolChi = false,
		hp = 0,
		todChiCost = 3,
		psKnown = false,
		willCapEnergy = false,
		spec = {},

		-- Energy, being the fast regenerating source it is, is a bad condition parameter
		-- So instead this function calculates the time needed to reach a given amount of energy.
		TimeToEnergy = function(self, req_energy)
			if(self.energy >= req_energy) then return 0 end

			return ((req_energy - self.energy) / self.energy_regen)
		end,
		AdjustEnergyRegen = function(self, s)
			local boostLeft = (self.energy_regen_boost_end - current_time)
			if boostLeft > 0 then
				if boostLeft >= s then
					self.energy_regen =  self.energy_regen_base + self.energy_regen_boost
				else
					self.energy_regen =  self.energy_regen_base + self.energy_regen_boost * (boostLeft / s)
				end
			end
		end,
		UseEnergy = function(self,e)
			self:UseTime(self:TimeToEnergy(e) + gcd_duration)
			self.energy = max(0, self.energy - e)
			self.willCapEnergy = self.energy == 100
		end,
		BoostEnergy = function(self, b)
			self.energy_regen = self.energy_regen + b -- temp boost energy regen for next actions calc
		end,
		UseTime = function(self, s)
			current_time = current_time + s
			self:AdjustEnergyRegen(s)
			self.energy = min(100, self.energy + s * self.energy_regen)
			self.willCapEnergy = self.energy == 100
		end,
		RefreshResources = function(self, s)
			local _
			self.spec = {}
			self.hp = 0
			self.poolChi = false
			self.chi = UnitPower("player", 12)
			self.energy = UnitPower("player")
			self.spell_haste = 1 + UnitSpellHaste("player") / 100
			_, self.energy_regen = GetPowerRegen()
			self.energy_regen_base = self.energy_regen
			self:UseTime(s)
		end,
		UseChi = function(self,c)
			self.chi = max(0, self.chi  - c)
		end,
		AddChi = function(self,c)
			self.chi = min(self.chiMax, self.chi  + c)
		end,
		GetPlayerHealthPercentage = function (self) -- minor optimization, since this is potentially called 4 times per cycle, and it doesnt really need to be that accurate
			if self.hp <= 0 then
				local player_health = UnitHealth("player")
				local player_health_max = UnitHealthMax("player")
				self.hp = (player_health / player_health_max) * 100
			end
			return self.hp
		end,
		IsPowerStrikesReady = function(self)
			if not self.psKnown then return false end
			
			local _, ps = GetTimeLeft("player", PowerStrikes, "HELPFUL|PLAYER")
			return ps
		end
	}

	-- Talent specific stuff
	local _, talent = GetTalentRowSelectionInfo(3)
	if talent == 7 then -- power strikes
		res.psKnown = true
	elseif talent == 8 then --ascension
		res.chiMax = 5
	end

	-- Glyph specific stuff
	if clcInfo_Monk.HasGlyphSocketed(123391) then
		res.todChiCost = 0
	end
	
	-- Lets each module do module specific stuff
	for _, mod in ipairs(clcInfo_Monk.modules) do
		if mod.RefreshLocals then
			mod.RefreshLocals()
		end
	end

	return res
end

-- called by each class module when clcinfo initializes, so here we do the initialization that is needed for each + that which needed when all have loaded
-- would have been cleaner with a OnInitComplete called by clcInfo for the latter, but since clcInfo does all its calls through the repective class modules (since taht is all it knows about) we do it this way
local modsLoaded = 0
function clcInfo_Monk:OnInitialize(class_module, db_default)
    local db = clcInfo:RegisterClassModuleDB(class_module.moduleName, db_default)
    local GetDB = self.newGetDB(db)
    local SetDB = self.newSetDB(db)
	
	-- Check is there is a template for it, and flag it as having been asked so we don't keep asking
	class_module.templateMissing = not (db.asked_missing or self.TemplateExistsFor(class_module.specIdx))
	db.asked_missing = true
	modsLoaded = modsLoaded + 1

	--Now this part only happens once all expected modules have been through here, i.e. it happens on the last class module to initialize
	if modsLoaded == #self.modules then
		modsLoaded = 0
		activeResources = InitResources()
		
		-- register for talent/glyph change since we cannot rely on OnTemplatesUpdate as it won't fire on glyph change since it doesn't change the active template
		if not self.EventFrame:IsEventRegistered("PLAYER_TALENT_UPDATE") then
			self.EventFrame:RegisterEvent("PLAYER_TALENT_UPDATE")
		end

		-- some abilities require that is is known when they are cast as opposed to simply suggested
		if not self.EventFrame:IsEventRegistered("UNIT_SPELLCAST_SUCCEEDED") then --TODO: presently only needed by windwalker
			self.EventFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
		end

		-- get player level and register for level updates if < max
		playerLevel = UnitLevel("player")
		playerMaxLevel = GetMaxPlayerLevel()
		if playerLevel < playerMaxLevel and not self.EventFrame:IsEventRegistered("SKILL_LINES_CHANGED") then
			self.EventFrame:RegisterEvent("SKILL_LINES_CHANGED")
		end

		--init and apply general options
		self:InitializeGeneralOptions()
		
		-- Check for missing templates
		local specNames = ""
		for _,v in ipairs(self.modules) do
			if v.templateMissing then
				local _, specialization_name = GetSpecializationInfo(v.specIdx)
				specNames = specNames..specialization_name.."\n"
			end
		end
		
		-- if one or more templates are missing, prompt the user. Otherwise if there is a new version notification, show that.
		if specNames ~= "" then
			StaticPopup_Show("CLCINFO_MONK_QUESTION_CREATE_MISSING_TEMPLATES", specNames)
		elseif not self.general_db.notifiedVersion or self.general_db.notifiedVersion < self.NOTIFY_VERSION then
			StaticPopup_Show("CLCINFO_MONK_VERSION_INFO")
			self.general_db.notifiedVersion = self.NOTIFY_VERSION
		end
	end

    return db, GetDB, SetDB
end

-- update the actions based on the prio settings the user provides
-- set the queue for the given spec
function clcInfo_Monk:CreateQueue(actions, prioList)
    -- unless the player is lower than max level, the queue is identical to the current priolist
	if playerLevel == playerMaxLevel then return prioList end

	-- filter away unknown abilities
	local q = {}
    for i, value in ipairs(prioList) do
        if IsSpellKnown(actions[value].id) or (string.lower(value) == "jab") then
			table.insert(q, value)
		end
    end
    return q
end

function clcInfo_Monk.GetActiveTemplateSpec()
	if not (clcInfo.activeTemplate and clcInfo.activeTemplate.spec) then
		return nil, nil
	end

	return clcInfo.activeTemplate.spec.primary, clcInfo.activeTemplate.options.udLabel
end

local function CleanPriorities(prioList, actions)
	local check = {}
	for i=#prioList,1,-1 do
		local value = prioList[i]
		if check[value] or not actions[value] then
			table.remove(prioList, i)
		else
			check[value] = value
		end
	end
end

-- register module and use default settings if version has changed
-- returns the template db, the queue, and the get and set functions for the
-- module options screen
-- called by clcInfo when templates update
function clcInfo_Monk:OnTemplatesUpdate(class_module, tdb_default)
	local activeTemplateSpec, activeTemplateName = self.GetActiveTemplateSpec()
	if activeTemplateSpec ~= class_module.specIdx then
		if not activeTemplateSpec and class_module.specIdx == GetSpecialization() then -- warn if there is no active template but only once and only if current spec is covered by clcMonk, i.e. not Mistweaver
			self.OutputMessage(L["There is no active template."], "warn")
		end
		return
	end
	
    local tdb = clcInfo:RegisterClassModuleTDB(class_module.moduleName, tdb_default)
    local GetTDB, SetTDB
    if tdb then
        if not tdb.version or tdb.version < self.VERSION then
            -- fix stuff
            clcInfo.AdaptConfigAndClean(class_module.moduleName, tdb, tdb_default)
            tdb.version = self.VERSION
        end

		--regardless of version, we want to ensure a clean and valid priority table
		local actions = class_module.actions
		CleanPriorities(tdb.priority, actions)

		class_module.tdb = tdb
        GetTDB = self.newGetTDB(tdb)
        SetTDB = self.newSetTDB(tdb)
	else
		class_module.tdb = nil
    end

	class_module:UpdateModQueue()
	self.OutputMessage(L["Active template is: "]..activeTemplateName)

    return tdb, GetTDB, SetTDB
end

-- initializes the template db and q based on command line arguments
function clcInfo_Monk:CmdLine(module, args)
	local activeTemplateSpec = self.GetActiveTemplateSpec()
	if activeTemplateSpec ~= module.specIdx then
		if activeTemplateSpec == nil then
			self.OutputMessage(L["There is no active template, commandline option %s is not available."]:format(module.cmdPrioArg), "warn")
		elseif activeTemplateSpec == GetSpecialization() then
			local _, activeSpecName = GetSpecializationInfo(GetSpecialization())
			self.OutputMessage(L["The currently active template is for %s, commandline option %s is not available."]:format(activeSpecName, module.cmdPrioArg), "warn")
		else
			local _, templateSpecName = GetSpecializationInfo(activeTemplateSpec)
			self.OutputMessage(L["The currently active template is for %s, commandline option %s is not available."]:format(templateSpecName..L[" (forced)"], module.cmdPrioArg), "warn")
		end
		return
	end

	local actions = module.actions
	local prioList = module.tdb.priority

    -- strip any 'none' and check number of args
	local maxPriorities = 0
	for key, action in pairs(actions) do
        maxPriorities = maxPriorities + 1
    end

	if #args > maxPriorities then
		self.OutputMessage(L["Too many cmdLine priorities. Max is: "] .. maxPriorities, 'err')
		return
	end

	-- add args to options
	wipe(prioList)
    for i, arg in ipairs(args) do
        prioList[i] = arg
    end

	CleanPriorities(prioList, actions)
    
    -- redo queue
    module:UpdateModQueue()
    clcInfo:UpdateOptions()

	local _, modSpecName = GetSpecializationInfo(module.specIdx)
	self.OutputMessage(L["%s priority changed."]:format(modSpecName))
	--SendChatMessage(L["%s priority changed."]:format(modSpecName),"WHISPER",nil,UnitName("player"))
end

-- checks to see if a glyph is socketed
function clcInfo_Monk.HasGlyphSocketed(glyphSpellId)
    for i = 1, NUM_GLYPH_SLOTS do
        local _, _, _, socketedGlyphSpellId = GetGlyphSocketInfo(i)
        if socketedGlyphSpellId == glyphSpellId then
            return true
        end
    end
    return false
end

-- figures out which ability should be used next
local function GetNextAction(actions, q)
    local n = #q
    local action_index

    -- parse once, get cooldowns, return first 0
    for i = 1, n do
        action_index = q[i]
        local action = actions[action_index]
        local cd
		cd = action.GetCD(cur_id, activeResources)
		--[[
		if clcInfo_debug.enabled then
			clcInfo_debug:AddBoth(action_index, cd)
		end
		--]]
        if cd == 0 then
            return action
        end
        action.cd = cd
    end
    
	if clcInfo_debug.enabled then
        clcInfo_debug:AddBoth("SECOND ROUND", "No 1st round match")
    end
    -- parse again, return min cooldown
    local minQ = 1
    local minCd = actions[q[1]].cd
    for i = 2, n do
        local action_cd = actions[q[i]].cd
        if minCd > action_cd then
            minCd = action_cd
            minQ = i
        end
    end

    return actions[q[minQ]]
end

	
-- determines the current and next abilities
local function MonkRotation(actions, tdb, q)
    -- initialize resources for next pair of actions
    -- current time and remaining gcd adjusted for any active channel, e.g. fof
    current_time = GetTime()

	local gcd_left
	if activeChannelEnds > current_time then
		gcd_left = activeChannelEnds - current_time
	else
		-- gcd value
		local start, gcd = GetSpellCooldown(gcdSpellId)
		if gcd > 0 then
			gcd_duration = gcd -- while the gcd duration is always fixed at 1 if gcd is active, this must be set as it will start out at 0, i.e. it cannot be set on load
		end

		gcd_left = start + gcd_duration - current_time
		if gcd_left > (1 - tdb.latencyAdjust) then return end
		if gcd_left < 0 then gcd_left = 0 end
	end
    
    --resources
	if clcInfo_debug.enabled then
		clcInfo_debug:Clear()
	end
	activeResources:RefreshResources(gcd_left)
	--[[
	if clcInfo_debug.enabled then
		clcInfo_debug:AddBoth("Current", "--------------------")
		clcInfo_debug:AddBoth("actionin", gcd_left)
		clcInfo_debug:AddBoth("energy", activeResources.energy)
		clcInfo_debug:AddBoth("chi", activeResources.chi)
		clcInfo_debug:AddBoth("psrdy", activeResources:IsPowerStrikesReady() and "|cffdee029yes|r" or "no" )
		clcInfo_debug:AddBoth("regenboostleft",  max(0, (activeResources.energy_regen_boost_end - current_time)))
		clcInfo_debug:AddBoth("capNrg", activeResources.willCapEnergy and "|cffdee029yes|r" or "no")
	end
	--]]
	cur_id = nil -- new round no cur id yet
    local action = GetNextAction(actions, q)
	cur_id = action.id
	--[[
	if clcInfo_debug.enabled then
		clcInfo_debug:AddBoth("capNrgPost", activeResources.willCapEnergy and "|cffdee029yes|r" or "no")
		clcInfo_debug:AddBoth("pool", activeResources.poolChi and "|cffdee029yes|r" or "no")
		clcInfo_debug:AddBoth("cur", action.info)
	end
	--]]
    if action.UpdateResources then
		action.UpdateResources(activeResources)
	end
	
	activeResources.poolChi = false --TODO: is this correct are do we want it to pass through?
    
	--[[
	if clcInfo_debug.enabled then
		clcInfo_debug:AddBoth("Next", "--------------------")
		clcInfo_debug:AddBoth("actionin", (current_time - GetTime()))
		clcInfo_debug:AddBoth("energy", activeResources.energy)
		clcInfo_debug:AddBoth("chi", activeResources.chi)
		clcInfo_debug:AddBoth("psrdy", activeResources:IsPowerStrikesReady() and "|cffdee029yes|r" or "no")
		clcInfo_debug:AddBoth("regenboostleft", max(0, (activeResources.energy_regen_boost_end - current_time)))
		clcInfo_debug:AddBoth("capNrg", activeResources.willCapEnergy and "|cffdee029yes|r" or "no")
		clcInfo_debug:AddBoth("pool", activeResources.poolChi and "|cffdee029yes|r" or "no")
	end
	--]]
    action = GetNextAction(actions, q)
	next_id = action.id
	--[[
	if clcInfo_debug.enabled then
		clcInfo_debug:AddBoth("capNrgPost", activeResources.willCapEnergy and "|cffdee029yes|r" or "no")
		clcInfo_debug:AddBoth("pool", activeResources.poolChi and "|cffdee029yes|r" or "no")
		clcInfo_debug:AddBoth("next", action.info)
	end
	--]]
end

local nextSkill, rangeCheck -- a quick fix until this can be sorted properly, clearly the arg to IconCurrent is not known in NextExec
local devName = clcInfo_Monk.spells["Jab"].name

-- Get current ability
function clcInfo_Monk.IconCurrent(actions, tdb, q)
    MonkRotation(actions, tdb, q)

	rangeCheck = tdb.rangePerSkill or devName
    if nextSkill then nextSkill:DoUpdate() end
	return IconSpell(cur_id, rangeCheck)
end

-- Determine Next ability
local function NextExec()
    return IconSpell(next_id, rangeCheck)
end

-- reset next ability
local function ExecCleanupNext()
    nextSkill = nil
end

-- Get Next ability
function clcInfo_Monk.IconNext()
    nextSkill = emod.___e
    nextSkill:SetScript("OnUpdate", nil)
    nextSkill.exec = NextExec
    nextSkill.ExecCleanup = ExecCleanupNext
end

local function IsJab(id)
	local jabIds = { 115687, 115693, 115695, 115698, 108557, 100780 }
	for _, j in ipairs(jabIds) do
		if j == id then return true end
	end
	return false
end

-----------------EVENTS------------------------------------------
-- event handling.
clcInfo_Monk.EventFrame = CreateFrame("Frame", "clcInfo_MonkFrame", UIParent)
clcInfo_Monk.EventFrame:Hide()
clcInfo_Monk.EventFrame:SetScript("OnEvent", function(self, event, ...)
    clcInfo_Monk[event](clcInfo_Monk, event, ...)
end)

local FistsOfFuryId = clcInfo_Monk.spells["Fists of Fury"].id -- mixing in a spec specific check here. While basically wated time on Brewmaster its still deemed beter than to factor out to modueles with another fucn call overhead
local EnergizingBrewId  = clcInfo_Monk.spells["Energizing Brew"].id
function clcInfo_Monk.UNIT_SPELLCAST_SUCCEEDED(self, event, ...)
	local unitId, _, _, _, spellId = ...

    if unitId == "player" then
		activeChannelEnds = 0
		if spellId == FistsOfFuryId then
			activeChannelEnds = GetTime() + 4 / activeResources.spell_haste
		elseif spellId == EnergizingBrewId then
			activeResources.energy_regen_boost = 10
			activeResources.energy_regen_boost_end = GetTime() + 6
		end
	end
end

function clcInfo_Monk.PLAYER_TALENT_UPDATE(self, event, ...)
	activeResources = InitResources()

	-- Since class modules are not reloaded if the active template does not change, we want to reload them here if there is no active template so that buttons and stuff can change to reflect the new spec
	if not clcInfo.activeTemplate and clcInfo_Options then
		clcInfo_Options:LoadClassModules()
		clcInfo:UpdateOptions()
	end
end

function clcInfo_Monk.SKILL_LINES_CHANGED(self, event, ...)
	local cause = ...
	if cause then return end -- we only want to react to a nil cause

	-- Support for levelling monks so that queue is updated when they learn new spells
	if playerLevel < playerMaxLevel then
		local curLevel = UnitLevel("player")
		if curLevel ~= playerLevel then
			playerLevel = curLevel

			if playerLevel == playerMaxLevel then
				self.EventFrame:UnregisterEvent("SKILL_LINES_CHANGED")
			end

			for _,v in ipairs(self.modules) do
				v:UpdateModQueue()
			end
		end
	end
end