﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_Brewmaster.lua
--
--    Copyright (C) 2012 Geminior
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  To a great extend this is based on the clcInfo_Rogue by Deronda.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Geminior (@ Earthen Ring)
------------------------------------------------------------------------------
-- Description:
--   Actions module for Brewmaster Monks
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Monk then return end -- need clcInfo_Monk
local _, englishClass = UnitClass("player") -- need to be a monk
if englishClass ~= clcInfo_Monk.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local min, max = min, max
local clcInfo_Monk = clcInfo_Monk
local spells = clcInfo_Monk.spells
local GetTimeLeft = clcInfo_Monk.GetTimeLeft
local GetCooldown = clcInfo_Monk.GetCooldown
local IconNext = clcInfo_Monk.IconNext
local IconCurrent = clcInfo_Monk.IconCurrent
					
--these are just initial values some of which will change on init
local class_module = {
	specIdx = 1,
	moduleName = "Brewmaster",
	cmdPrioArg = "brewmasterprio",
}

local db_default = {}
local db

-- Assign module variables
local default_priority = {"tod","hs","grd","bok","bof","ehlh","ks","eh","tpow","jab","tp"}
local tdb_default = {
    version = clcInfo_Monk.VERSION,
    priority = default_priority,
    rangePerSkill = false,
	latencyAdjust = 0.2,
	tpRefresh = 2,
	bokRefresh = 6,
	purStaggerUntil = 4,
	todWait = 4,
	poolTod = true,
	useElubAt = 3,
	waitKs = 0.8
}
local tdb

-- actions ---------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action functions up
local LightStagger = spells["Light Stagger"].name
local ModerateStagger = spells["Moderate Stagger"].name
local HeavyStagger = spells["Heavy Stagger"].name
local Shuffle = spells["Shuffle"].name
local ElusiveBrew = spells["Elusive Brew"].name
local TigerPower = spells["Tiger Power"].name
local DeathNote = spells["Death Note"].name
local BreathOfFire = spells["Breath of Fire"].name

local KegSmashId = spells["Keg Smash"].id
local GuardId = spells["Guard"].id
local TigerPalmId = spells["Tiger Palm"].id
local BlackoutKickId = spells["Blackout Kick"].id
local JabId = spells["Jab"].id
local TouchOfDeathId = spells["Touch of Death"].id
local ExpelHarmId = spells["Expel Harm"].id
local ProvokeId = spells["Provoke"].id
local BreathOfFireId = spells["Breath of Fire"].id
local PurifyingBrewId = spells["Purifying Brew"].id
local ElusiveBrewId = spells["Elusive Brew"].id

local L = LibStub("AceLocale-3.0"):GetLocale("clcInfo_Monk")

-- This actually ought to be a function, so that any spell changes occuring due to glyphs etc. would also be updated in the id of each action. But since the id's of such glyphed spells return the same icon as non glyphed I won't bother
local actions  = {
	tod = {
		id = TouchOfDeathId,
		GetCD = function(cur_id, r)
			if cur_id == TouchOfDeathId then
				return 100
			end
			
			local _, canTod = GetTimeLeft("player", DeathNote, "HELPFUL|PLAYER")
			if not canTod then
				return 100
			end
			if r.chi >= r.todChiCost then
				return GetCooldown(TouchOfDeathId, -tdb.todWait)
			end

			if(tdb.poolTod) then -- if we can Tod now or within the acceptable wait time but lack the chi, pool chi and let chi builders at it
				r.poolChi = GetCooldown(TouchOfDeathId, -tdb.todWait) <= 0
			end
			return 100
		end,
		UpdateResources = function(r)
			r:UseChi(r.todChiCost)
			r:UseTime(1)
		end,
		info = spells["Touch of Death"].name
	},
	bof = {
		id = BreathOfFireId,
		GetCD = function(cur_id, r)		
			if cur_id == BreathOfFireId or r.poolChi or r.chi < 2 or ((r.chi <= r.chiMax - 2) and r.willCapEnergy) then
				return 100
			end
			
			return (GetTimeLeft("target", BreathOfFire, "HARMFUL"))
		end,
		UpdateResources = function(r)
			r:UseChi(2)
			r:UseTime(1)
		end,
		info = spells["Breath of Fire"].name
	},
	hs = {
		id = PurifyingBrewId,
		GetCD = function(cur_id, r)		

			if cur_id == PurifyingBrewId or r.poolChi or r.chi < 1 then
				return 100
			end
			
			if (GetTimeLeft("player", HeavyStagger, "HARMFUL")) > tdb.purStaggerUntil then
				return 0
			end
			
			return 100
		end,
		UpdateResources = function(r)
			r:UseChi(1)
		end,
		info = L["Purify %s"]:format(HeavyStagger)
	},
	ms = {
		id = PurifyingBrewId,
		GetCD = function(cur_id, r)		

			if cur_id == PurifyingBrewId or r.poolChi or r.chi < 1 then
				return 100
			end
			
			if (GetTimeLeft("player", ModerateStagger, "HARMFUL")) > tdb.purStaggerUntil then
				return 0
			end
			
			return 100
		end,
		UpdateResources = function(r)
			r:UseChi(1)
		end,
		info = L["Purify %s"]:format(ModerateStagger)
	},
	ls = {
		id = PurifyingBrewId,
		GetCD = function(cur_id, r)		

			if cur_id == PurifyingBrewId or r.poolChi or r.chi < 1 then
				return 100
			end
			
			if (GetTimeLeft("player", LightStagger, "HARMFUL")) > tdb.purStaggerUntil then
				return 0
			end
			
			return 100
		end,
		UpdateResources = function(r)
			r:UseChi(1)
		end,
		info = L["Purify %s"]:format(LightStagger)
	},
	tpow = {
		id = TigerPalmId,
		GetCD = function(cur_id, r)
			if  cur_id == TigerPalmId or r.poolChi then --yes this doesn't use chi but doesn't generate it either
				return 100
			end
			return max(0, (GetTimeLeft("player", TigerPower, "HELPFUL|PLAYER")) - tdb.tpRefresh)
		end,
		UpdateResources = function(r)
			r:UseTime(1)
		end,
		info = spells["Tiger Power"].name
	},
	tp = {
		id = TigerPalmId,
		GetCD = function(cur_id, r)
			return 0.7
		end,
		UpdateResources = function(r)
			r:UseTime(1)
		end,
		info = spells["Tiger Palm"].name
	},
	bok = {
		id = BlackoutKickId,
		GetCD = function(cur_id, r)
			if r.chi < 2 or r.poolChi or ((r.chi <= r.chiMax - 2) and r.willCapEnergy) then
				return 100
			end

			-- return time left on shuffle, to allow other chi consumers in if shuffle is at a comfortable level
			return max(0, (GetTimeLeft("player", Shuffle, "HELPFUL|PLAYER")) - tdb.bokRefresh) 
		end,
		UpdateResources = function(r)
			r:UseChi(2)
			r:UseTime(1)
		end,
		info = spells["Blackout Kick"].name
	},
	grd = {
		id = GuardId,
		GetCD = function(cur_id, r)
			if cur_id == GuardId or r.chi < 2 or r.poolChi or ((r.chi <= r.chiMax - 2) and r.willCapEnergy) then
				return 100
			end
			
			return GetCooldown(GuardId)
		end,
		UpdateResources = function(r)
			r:UseChi(2)
		end,
		info = spells["Guard"].name
	},
	eh = {
		id = ExpelHarmId,
		GetCD = function(cur_id, r)
			if cur_id ~= ExpelHarmId and r.chi < r.chiMax and r:GetPlayerHealthPercentage() < 90 then
				return max(r:TimeToEnergy(40), GetCooldown(ExpelHarmId))
			end
			
			return 100
		end,
		UpdateResources = function(r)
			r:AddChi(1)
			r:UseEnergy(40)
		end,
		info = spells["Expel Harm"].name,
	},
	ehlh = {
		id = ExpelHarmId,
		GetCD = function(cur_id, r)
			if r.chi < r.chiMax and r:GetPlayerHealthPercentage() < 35 then
				return max(r:TimeToEnergy(40), GetCooldown(ExpelHarmId))
			end
			
			return 100
		end,
		UpdateResources = function(r)
			r:AddChi(1)
			r:UseEnergy(40)
		end,
		info = spells["Expel Harm"].name .. " < 35% HP",
	},
	jab = {
		id = JabId,
		GetCD = function(cur_id, r)
			return max(r:TimeToEnergy(40), GetCooldown(JabId))
		end,
		UpdateResources = function(r)
			local chiGain = r:IsPowerStrikesReady() and 2 or 1
			r:AddChi(chiGain)
			r:UseEnergy(40)
		end,
		info = spells["Jab"].name,
	},
	ks = {
		id = KegSmashId,
		GetCD = function(cur_id, r)
			if cur_id ~= KegSmashId and r.chi < (r.chiMax - 1) then
				return max(r:TimeToEnergy(40), GetCooldown(KegSmashId, -tdb.waitKs))
			end
			return 100
		end,
		UpdateResources = function(r)
			r:AddChi(2)
			r:UseEnergy(40)
		end,
		info = spells["Keg Smash"].name,
	},
	eb = {
		id = ElusiveBrewId,
		GetCD = function(cur_id, r)
			if cur_id == ElusiveBrewId then
				return 100
			end
			
			local _, stacks = GetTimeLeft("player", ElusiveBrew, "HELPFUL|PLAYER")
			if stacks ~= nil and stacks >= tdb.useElubAt then
				return GetCooldown(ElusiveBrewId)
			end
			return 100
		end,
		info = spells["Elusive Brew"].name,
	},
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB

--initialize module using the defaults set ^
class_module = clcInfo_Monk:RegisterModule(class_module)
class_module.recommendedPriorities = {
	[L["Solo"]] = function(self)
		return function()
			local tdb = self.tdb
			tdb.tpRefresh = 2
			tdb.priority = {"tod","tpow","grd","bok","bof","ehlh","ks","eh","jab","tp"}
			self:UpdateModQueue()
			clcInfo:UpdateOptions()
		end
	end,
	[L["Raid / Group"]] = function(self)
		return function()
			local tdb = self.tdb
			tdb.tpRefresh = 2
			tdb.priority = {"tod","hs","ms","grd","bok","bof","ehlh","ks","eh","tpow","jab","tp"}
			self:UpdateModQueue()
			clcInfo:UpdateOptions()
		end
	end
}
class_module.actions = actions

--called when template updates.
function class_module.OnTemplatesUpdate()
    tdb, GetTDB, SetTDB = clcInfo_Monk:OnTemplatesUpdate(class_module, tdb_default)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db, GetDB, SetDB = clcInfo_Monk:OnInitialize(class_module, db_default)
end

function class_module:UpdateModQueue()
	if self.tdb then
		self.q = clcInfo_Monk:CreateQueue(actions, self.tdb.priority)
	else
		self.q = nil
	end
end

-- updates the id of the jab action so that the selected/weapon specific jab icon is shown
function class_module.SetJabActionId(id)
	actions.jab.id = id
end

function class_module.RefreshLocals()
	-- The Guard glyph changes the spell id of Guard
	local grdId = 115295
	if clcInfo_Monk.HasGlyphSocketed(123401) then
		grdId = 123402
	end
	clcInfo_Monk:SetSpellInfo("Guard", grdId)
	GuardId = spells["Guard"].id
end

-- function called by clcInfo on first load (empty saved vars)
function class_module:CreateDefaultTemplate()
	return clcInfo_Monk.CreateTemplate(self)
end

-- called if a default template should be created, lots of back and forth going on here.... Oninitialize is call by clcInfo, which in turn calls OnInitialize on clcInfo_Monk which then calls this as needed
-- but we cannot do this directly in OnInitialize since that is called once per module and we only want to ask the user about default templates once
-- This returns the icons needed by the module
function class_module.CreateDefaultTemplateIcons(gridIdx)
	local icons = clcInfo.display.icons

	--current action
	local icon1 = icons:GetDefault()
	icon1.udLabel = "IconCurrent"
	icon1.skinSource = "Grid"
	icon1.gridId = gridIdx
	icon1.gridX = 1
	icon1.exec = "return IconBrewmasterCurrent()"

	--next action
	local icon2 = icons:GetDefault()
	icon2.udLabel = "IconNext"
	icon2.skinSource = "Grid"
	icon2.gridId = gridIdx
	icon2.gridX = 2
	icon2.exec = "IconBrewmasterNext()"

	--eulisve brew
	local icon3 = icons:GetDefault()
	icon3.udLabel = "IconBrew"
	icon3.skinSource = "Grid"
	icon3.gridId = gridIdx
	icon3.gridX = 3
	icon3.ups = 3
	icon3.exec = "return ElusiveBrewIcon()"

	return { icon1, icon2, icon3 }
end

-- called when module options are opened
local function LoadBrewmasterModule()
    clcInfo_Monk.LoadModule(class_module, GetDB, SetDB, GetTDB, SetTDB)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadBrewmasterModule

-- giving queue as command line
local function CmdBrewmaster(args)
    clcInfo_Monk:CmdLine(class_module, args)
end

clcInfo.cmdList[class_module.cmdPrioArg] = CmdBrewmaster

----ABILITY ICONS--------------------------------------------------------------

----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
local IconSpell = clcInfo.env.IconSpell
function emod.IconBrewmasterCurrent(...)
    return IconCurrent(actions, tdb, class_module.q)
end
function emod.IconBrewmasterNext(...)
    return IconNext()
end
function emod.ElusiveBrewIcon()
	local _, stacks = GetTimeLeft("player", ElusiveBrew, "HELPFUL|PLAYER")
	if stacks ~= nil and stacks >= tdb.useElubAt and GetCooldown(ElusiveBrewId) == 0 then
		return IconSpell(ElusiveBrewId, false)
	end
end