﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_MonkGeneral.lua
--
--    Copyright (C) 2012 Geminior
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Geminior (@ Earthen Ring)
------------------------------------------------------------------------------
-- Description:
--   General Settings for Monks
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Monk then return end -- need clcInfo_Monk
local _, englishClass = UnitClass("player") -- need to be a monk
if englishClass ~= clcInfo_Monk.CLASS then return end

local L = LibStub("AceLocale-3.0"):GetLocale("clcInfo_Monk")

local moduleName = "clcMonk_GeneralOptions"
local general_db_defaults = {
	jabIconId = 115693,
	jabIconAuto = true
}

local jabLookup
local function GetJabTextureLookup()
	if jabLookup then return jabLookup end

	jabLookup = {}
	jabLookup[L["Axe"]] =		{ id = 115687, texture="INTERFACE\\ICONS\\inv_axe_97" }
	jabLookup[L["Mace"]] =		{ id = 115693, texture="Interface\\Icons\\inv_mace_08" }
	jabLookup[L["Sword"]] =		{ id = 115695, texture="Interface\\Icons\\INV_Sword_10" }
	jabLookup[L["Polearm"]] =	{ id = 115698, texture="Interface\\Icons\\INV_Spear_03" }
	jabLookup[L["Staff"]] =		{ id = 108557, texture="Interface\\Icons\\ability_monk_staffstrike" }
	jabLookup[L["Fist"]] =		{ id = 108967, texture="Interface\\Icons\\ability_monk_jab" } -- this isn't a monk spell but it has the jab fist icon, whereas 100780 has a dynamic icon

	return jabLookup
end

local function UpdateJabActionId()
	local jabId
	if clcInfo_Monk.general_db.jabIconAuto then
		jabId = clcInfo_Monk.spells["Jab"].id
	else
		jabId = clcInfo_Monk.general_db.jabIconId or clcInfo_Monk.spells["Jab"].id -- just to be on the safe side
	end

	for _, mod in ipairs(clcInfo_Monk.modules) do
		mod.SetJabActionId(jabId)
	end
end

function clcInfo_Monk:InitializeGeneralOptions()
	self.general_db = clcInfo:RegisterClassModuleDB(moduleName, general_db_defaults)
	UpdateJabActionId()
end

local function GetJabAutoValue (info, val)
    return clcInfo_Monk.general_db[info[#info]]
end

local function SetJabAutoValue (info, val)
    clcInfo_Monk.general_db[info[#info]] = val
	UpdateJabActionId()
end

local function GetJabIconValue (info, val)
	local jabId = clcInfo_Monk.general_db[info[#info]]
	for k,v in pairs(GetJabTextureLookup()) do
		if v.id == jabId then return k end
	end
    return L["Axe"]
end

local function SetJabIconValue (info, val)
	for k,v in pairs(GetJabTextureLookup()) do
		if k == val then
			clcInfo_Monk.general_db[info[#info]] = v.id
			break
		end
	end
	UpdateJabActionId()
end

local function ToggleJabIconEnabled()
	return clcInfo_Monk.general_db.jabIconAuto
end

local cfgMatrix = {
    order = 1, type = "group", childGroups = "tab", name = L["Monk General Options"],
    args = {
        tabOptions = {
            order = 1, type = "group", name = L["Options"], args = {
                igSettings = {
                    order=1, type = "group", inline = true, name=L["Jab Icon Settings"], args = {
                        jabIconAuto = {
                            order = 2, type = "toggle",
                            name = L["Automatic"],
							desc = L["If checked, the Jab icon will be based on the current weapon used."], 
                            get = GetJabAutoValue, set = SetJabAutoValue,
						},
						jabIconId = {
                            order = 4, type = "select", dialogControl = 'CLCM_SpellIcon', values = GetJabTextureLookup,
                            name = L["Specific Icon"], desc = L["Select an icon to always display for Jab. This is ignored if Automatic is checked"],
							get = GetJabIconValue, set = SetJabIconValue, disabled = ToggleJabIconEnabled,
                        },
                    },
                },
            },           
        },
    },
}
-- called when module options are opened
local function LoadGeneralOptions()
    clcInfo_Options.options.args.classModules.args[moduleName] = cfgMatrix
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadGeneralOptions