﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_Windwalker.lua
--
--    Copyright (C) 2012 Geminior
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  To a great extend this is based on the clcInfo_Rogue by Deronda.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Geminior (@ Earthen Ring)
------------------------------------------------------------------------------
-- Description:
--   Actions module for Windwalker Monks
------------------------------------------------------------------------------
if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Monk then return end -- need clcInfo_Monk
local _, englishClass = UnitClass("player") -- need to be a monk
if englishClass ~= clcInfo_Monk.CLASS then return end

-- Get local copies of functions/tables.  This will speed up execution.
local min, max = min, max
local clcInfo_Monk = clcInfo_Monk
local spells = clcInfo_Monk.spells
local GetTimeLeft = clcInfo_Monk.GetTimeLeft
local GetCooldown = clcInfo_Monk.GetCooldown
local IconNext = clcInfo_Monk.IconNext
local IconCurrent = clcInfo_Monk.IconCurrent
local clcInfo_debug = clcInfo.debug

--these are just initial values some of which will change on init
local class_module = {
	specIdx = 3,
	moduleName = "Windwalker",
	cmdPrioArg = "windwalkerprio",
}

local db_default = {}
local db

-- Assign module variables
local default_priority = {"tod","rskdb","tpow","eb","rsk","fof","cbbok","bok","cbtp","eh","jab", "fsk"}
local tdb_default = {
    version = clcInfo_Monk.VERSION,
    priority = default_priority,
    rangePerSkill = false,
	tpRefresh = 4,
	latencyAdjust = 0.2,
	useEbTh = 5,
	useTebAt = 10,
	todWait = 4,
	poolTod = true,
	poolFof = false,
	rskMaxDelay = 0.5
}
local tdb

-- actions ---------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
local RisingSunKick = spells["Rising Sun Kick"].name
local ComboBreakerBlackoutKick = spells["Combo Breaker: Blackout Kick"].name
local ComboBreakerTigerPalm = spells["Combo Breaker: Tiger Palm"].name
local TigerPower = spells["Tiger Power"].name
local TigereyeBrew = spells["Tigereye Brew"].name
local DeathNote = spells["Death Note"].name

local RisingSunKickId = spells["Rising Sun Kick"].id
local FistsOfFuryId = spells["Fists of Fury"].id
local EnergizingBrewId = spells["Energizing Brew"].id
local TigereyeBrewId = spells["Tigereye Brew"].id
local TigerPalmId = spells["Tiger Palm"].id
local BlackoutKickId = spells["Blackout Kick"].id
local JabId = spells["Jab"].id
local TouchOfDeathId = spells["Touch of Death"].id
local ExpelHarmId = spells["Expel Harm"].id
local FlyingSerpentKickId = spells["Flying Serpent Kick"].id

local L = LibStub("AceLocale-3.0"):GetLocale("clcInfo_Monk")

local actions  = {
	tod = {
		id = TouchOfDeathId,
		GetCD = function(cur_id, r)
			if cur_id == TouchOfDeathId then
				return 200
			end
			
			local _, canTod = GetTimeLeft("player", DeathNote, "HELPFUL|PLAYER")
			if not canTod then
				return 200
			end
			if r.chi >= r.todChiCost then
				return GetCooldown(TouchOfDeathId, -tdb.todWait)
			end

			if(tdb.poolTod) then -- if we can Tod now or within the acceptable wait time but lack the chi, pool chi and let chi builders at it
				r.poolChi = GetCooldown(TouchOfDeathId, -tdb.todWait) <= 0
			end
			return 100
		end,
		UpdateResources = function(r)
			r:UseChi(r.todChiCost)
			r:UseTime(1)
		end,
		info = spells["Touch of Death"].name
	},
	fof = {
		id = FistsOfFuryId,
		GetCD = function(cur_id, r)
			if cur_id == FistsOfFuryId then
				return 200
			end
			
			-- if we aren't off cd we don't want to do the calculations below
			local fofcd = GetCooldown(FistsOfFuryId)
			if fofcd > 0 then
				return 100 + fofcd
			end
			
			-- ok we so if fof will cause us to cap energy (runs 4 secs) we dont want it, if we will loose TigerPower we don't want it
			-- if RSK will be delayed (including the 1 GCD needed or jav to get back enough chi) we don't want it. Thre is no need to take energy for jab into account since even at 0 energy a full FOF will last long enough to get to 40 energy before its done
			if r.chi > 2 then
				local fofTime = 4 / r.spell_haste
				if (GetTimeLeft("player", TigerPower, "HELPFUL|PLAYER")) < fofTime + 1 or r:TimeToEnergy(100) < fofTime or GetCooldown(RisingSunKickId) < fofTime + 1 - tdb.rskMaxDelay then
					return 101
				end
				return 0
			end

			-- ok so do we pool chi? this calculation is then for the next round, so we need to adjust to the expected situation 1 gcd ahead
			-- the expectation is that fof will effectively use 5 seconds (4 + the 1 gcd) and energy will be 40 less than now, hence 140
			-- of course that requires that we actually have 40 energy, if we have atleast 1 chi there are better things to do otherwise it will pool anyway
			if tdb.poolFof then
				local fofTime = 4 / r.spell_haste
				r.poolChi = (GetTimeLeft("player", TigerPower, "HELPFUL|PLAYER")) >= fofTime + 2 and r:TimeToEnergy(140) >= fofTime and GetCooldown(RisingSunKickId) > fofTime + 2 - tdb.rskMaxDelay -- the +2 bit is just to add a resonable time for the user to be able to refresh tp, 1 GCD + 1 sec
			end
			return 102
		end,
		UpdateResources = function(r)
			r:UseChi(3)
			r:UseTime(4 / r.spell_haste)
		end,
		info = spells["Fists of Fury"].name
	},
	rskdb = {
		id = RisingSunKickId,
		GetCD = function(cur_id, r)
			if cur_id == RisingSunKickId or r.chi < 2 or r.poolChi then
				return 100
			end

			if (GetTimeLeft("target", RisingSunKick, "HARMFUL")) > 0 then
				return 100
			end
			
			return GetCooldown(RisingSunKickId)
		end,
		UpdateResources = function(r)
			r:UseChi(2)
			r:UseTime(1)
		end,
		info = spells["Rising Sun Kick"].name .. " " .. L["Debuff"]
	},
	rsk = {
		id = RisingSunKickId,
		GetCD = function(cur_id, r)
			if cur_id == RisingSunKickId or r.chi < 2 or r.poolChi then
				return 100
			end

			local rskCd = GetCooldown(RisingSunKickId)
			-- if at 4 or more chi we have the chi to do 2 consecutive chi consumers, i.e. we can do a bok before rsk as long as it won't delay rsk more than the configured amount
			if r.chi >= 4 then
				local delay = max(0, 1 - rskCd)
				if delay > tdb.rskMaxDelay then
					return 0
				else
					return rskCd
				end
			end
			
			-- so we have either 2 or 3 chi if we get here which means that using another chi consumer will incur
			-- one additional GCD or the time needed to get 40 energy to jab, whichever is higher + 1 GCD for the Jab
			-- if this exceeds the configured max delay, rsk is suggested even if on CD
			if max(1, r:TimeToEnergy(40)) + 1 - rskCd > tdb.rskMaxDelay then
				return 0
			end

			return rskCd
		end,
		UpdateResources = function(r)
			r:UseChi(2)
			r:UseTime(1)
		end,
		info = spells["Rising Sun Kick"].name
	},
	tpow = {
		id = TigerPalmId,
		GetCD = function(cur_id, r)
			if (r.chi < 1 or r.poolChi) then -- TODO: poolchi should only be honored if its for tod, not fof
				return 100
			end

			if cur_id == TigerPalmId then
				return 100
			end

			return max(0, (GetTimeLeft("player", TigerPower, "HELPFUL|PLAYER")) - tdb.tpRefresh)
		end,
		UpdateResources = function(r)
			-- If prioritized above cbtp and cbtp procs but tiger power buff is about to expire then this may well be suggested but in fact it will be a cbtp when used
			if ((GetTimeLeft("player", ComboBreakerTigerPalm, "HELPFUL|PLAYER")) > 0) then
				r.spec.cbtp = true
				r:UseTime(1)
			else
				r:UseChi(1)
				r:UseTime(1)
			end
		end,
		info = spells["Tiger Power"].name
	},
	tp = {
		id = TigerPalmId,
		GetCD = function(cur_id, r)
			if r.chi < 1 or r.poolChi then
				return 100
			end
			if ((r.chi <= r.chiMax - 2) and r.willCapEnergy) then
				return 101
			end
			return GetCooldown(TigerPalmId)
		end,
		UpdateResources = function(r)
			r:UseChi(1)
			r:UseTime(1)
		end,
		info = spells["Tiger Palm"].name
	},
	cbtp = {
		id = TigerPalmId,
		GetCD = function(cur_id, r)
			if r.spec.cbtp or (GetTimeLeft("player", ComboBreakerTigerPalm, "HELPFUL|PLAYER")) <= 0 then
				return 100
			end
			return GetCooldown(TigerPalmId)
		end,
		UpdateResources = function(r)
			r.spec.cbtp = true
			r:UseTime(1)
		end,
		info = spells["Combo Breaker: Tiger Palm"].name
	},
	bok = {
		id = BlackoutKickId,
		GetCD = function(cur_id, r)
			if r.chi < 2 or r.poolChi then
				return 100
			end

			if ((r.chi <= r.chiMax - 2) and r.willCapEnergy) then
				return 101
			end
			
			return GetCooldown(BlackoutKickId)
		end,
		UpdateResources = function(r)
			r:UseChi(2)
			r:UseTime(1)
		end,
		info = spells["Blackout Kick"].name
	},
	cbbok = {
		id = BlackoutKickId,
		GetCD = function(cur_id, r)
			if r.spec.cbbok or (GetTimeLeft("player", ComboBreakerBlackoutKick, "HELPFUL|PLAYER")) <= 0 then
				return 100
			end

			return GetCooldown(BlackoutKickId)
		end,
		UpdateResources = function(r)
			r.spec.cbbok = true
			r:UseTime(1)
		end,
		info = spells["Combo Breaker: Blackout Kick"].name
	},
	eh = {
		id = ExpelHarmId,
		GetCD = function(cur_id, r)
			if cur_id ~= ExpelHarmId and r:GetPlayerHealthPercentage() < 90 then
				return max(r:TimeToEnergy(38), GetCooldown(ExpelHarmId))
			end
			
			return 100
		end,
		UpdateResources = function(r)
			r:AddChi(2)
			r:UseEnergy(40) -- time use included
		end,
		info = spells["Expel Harm"].name,
	},
	jab = {
		id = JabId,
		GetCD = function(cur_id, r)
			return max(r:TimeToEnergy(38), GetCooldown(JabId))
		end,
		UpdateResources = function(r)
			local chiGain = r:IsPowerStrikesReady() and 3 or 2
			r:AddChi(chiGain)
			r:UseEnergy(40) -- time use included
		end,
		info = spells["Jab"].name,
	},
	eb = {
		id = EnergizingBrewId,
		GetCD = function(cur_id, r)
			if cur_id == EnergizingBrewId or (r:TimeToEnergy(100) < tdb.useEbTh) then
				return 100
			end
			
			return GetCooldown(EnergizingBrewId)
		end,
		UpdateResources = function(r)
			r:BoostEnergy(10)
		end,
		info = spells["Energizing Brew"].name,
	},
	teb = {
		id = TigereyeBrewId,
		GetCD = function(cur_id, r)
			if cur_id == TigereyeBrewId then
				return 100
			end

			local t, stacks = GetTimeLeft("player", TigereyeBrew, "HELPFUL|PLAYER")
			if (stacks and (stacks >= tdb.useTebAt or (stacks > 0 and t < 4))) then -- the buff is the same name when active but with 0 stacks hence the latter check
				return GetCooldown(TigereyeBrewId)
			end

			return 100
		end,
		info = spells["Tigereye Brew"].name,
	},
	fsk = {
		id = FlyingSerpentKickId,
		GetCD = function(cur_id, r)
			if cur_id == FlyingSerpentKickId then
				return 0
			end
			return GetCooldown(FlyingSerpentKickId) + 1.5 --yeah so hardcoded this one. Basically we don't want to use this unless all other options are unavailable for atleast the time it takes to get this off and land again
		end,
		info = spells["Flying Serpent Kick"].name,
	},
	
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB

--initialize module using the defaults set ^
class_module = clcInfo_Monk:RegisterModule(class_module)
class_module.recommendedPriorities = {
	[L["Solo"]] = function(self)
		return function()
			local tdb = self.tdb
			tdb.poolFof = false
			tdb.tpRefresh = 2
			tdb.priority = {"tod","rskdb","tpow","eb","rsk","fof","cbbok","bok","cbtp", "eh","jab", "fsk"}
			self:UpdateModQueue()
			clcInfo:UpdateOptions()
		end
	end,
	[L["Raid / Group"]] = function(self)
		return function()
			local tdb = self.tdb
			tdb.poolFof = false
			tdb.tpRefresh = 2
			tdb.priority = {"tod","rskdb","tpow","eb","rsk","fof","cbbok","bok","cbtp","jab", "fsk"}
			self:UpdateModQueue()
			clcInfo:UpdateOptions()
		end
	end
}
class_module.actions = actions

--called when template updates.
function class_module.OnTemplatesUpdate()
    tdb, GetTDB, SetTDB = clcInfo_Monk:OnTemplatesUpdate(class_module, tdb_default)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db, GetDB, SetDB = clcInfo_Monk:OnInitialize(class_module, db_default)
end

function class_module:UpdateModQueue()
	if self.tdb then
		self.q = clcInfo_Monk:CreateQueue(actions, self.tdb.priority)
	else
		self.q = nil
	end
end

-- updates the id of the jab action so that the selected/weapon specific jab icon is shown
function class_module.SetJabActionId(id)
	actions.jab.id = id
end

-- function called by clcInfo on first load (empty saved vars)
function class_module:CreateDefaultTemplate()
	return clcInfo_Monk.CreateTemplate(self)
end

-- called if a default template should be created, lots of back and forth going on here.... Oninitialize is call by clcInfo, which in turn calls OnInitialize on clcInfo_Monk which then calls this as needed
-- but we cannot do this directly in OnInitialize since that is called once per module and we only want to ask the user about default templates once
-- This returns the icons needed by the module
function class_module.CreateDefaultTemplateIcons(gridIdx)
	local icons = clcInfo.display.icons

	--current action
	local icon1 = icons:GetDefault()
	icon1.udLabel = "IconCurrent"
	icon1.skinSource = "Grid"
	icon1.gridId = gridIdx
	icon1.gridX = 1
	icon1.exec = "return IconWindwalkerCurrent()"

	--next action
	local icon2 = icons:GetDefault()
	icon2.udLabel = "IconNext"
	icon2.skinSource = "Grid"
	icon2.gridId = gridIdx
	icon2.gridX = 2
	icon2.exec = "IconWindwalkerNext()"

	--tiger eye brew
	local icon3 = icons:GetDefault()
	icon3.udLabel = "IconBrew"
	icon3.skinSource = "Grid"
	icon3.gridId = gridIdx
	icon3.gridX = 3
	icon3.ups = 3
	icon3.exec = "return TigerEyeBrewIcon()"


	return { icon1, icon2, icon3 }
end

-- called when module options are opened
local function LoadWindwalkerModule()
    clcInfo_Monk.LoadModule(class_module, GetDB, SetDB, GetTDB, SetTDB)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadWindwalkerModule

-- giving queue as command line
local function CmdWindwalker(args)
    clcInfo_Monk:CmdLine(class_module, args)
end

clcInfo.cmdList[class_module.cmdPrioArg] = CmdWindwalker

----ABILITY ICONS--------------------------------------------------------------

----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
local IconSpell = clcInfo.env.IconSpell
function emod.IconWindwalkerCurrent(...)
    return IconCurrent(actions, tdb, class_module.q)
end
function emod.IconWindwalkerNext(...)
    return IconNext()
end
function emod.TigerEyeBrewIcon()
	local t, stacks = GetTimeLeft("player", TigereyeBrew, "HELPFUL|PLAYER")
	if stacks and (stacks >= tdb.useTebAt or (stacks > 0 and t < 4)) and GetCooldown(TigereyeBrewId) == 0 then
		return IconSpell(TigereyeBrewId, false)
	end
end