﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                                options.lua
--
--    Copyright (C) 2012 Geminior
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.  To a great extend this is based on the clcInfo_Rogue by Deronda.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      Geminior (@ Earthen Ring)
------------------------------------------------------------------------------
-- Description:
--   Options for clcInfo_Monk
------------------------------------------------------------------------------
if not clcInfo then return end --clcInfo must be loaded
if not clcInfo_Monk then return end

local _, englishClass = UnitClass("player")
if englishClass ~= clcInfo_Monk.CLASS then return end  --must be monk

--each spec needs its own set and get functions to refer to their
--own db and tdb upvalues.  So we use closures for Get and Set functions for
--the AceGUI callbacks.
function clcInfo_Monk.newGetDB(db)
    return function (info)
               return db[info[#info]]
           end
end

function clcInfo_Monk.newSetDB(db)
    return function (info, val)
               db[info[#info]] = val
           end
end

function clcInfo_Monk.newGetTDB(tdb)
    return function (info)
               return tdb[info[#info]]
           end
end

function clcInfo_Monk.newSetTDB(tdb)
    return function (info, val)
               tdb[info[#info]] = val
           end
end

local function GetPriorities()
	return "tdb"
end

local function GetCreateTemplateFunc(module)
	return function () clcInfo_Monk:AddTemplate(module) end
end

local function GetChangeSpecFunc(class_module)
	local specGrp = clcInfo_Monk.GetSpecGroup(class_module)
	return function () SetActiveSpecGroup(specGrp) end
end

local L = LibStub("AceLocale-3.0"):GetLocale("clcInfo_Monk")

local function CreatePrioMacro(module)

	-- make sure there are at least one available slot
	local _, numperchar = GetNumMacros()
	if numperchar == 18 then --max per char macroes
		StaticPopup_Show("CLCINFO_MONK_MACRO_CREATE_ERROR", L["You have used all your character specific macro slots."])
		return
	end
	
	-- set the default macro name
	local _, specialization_name = GetSpecializationInfo(module.specIdx)
	local tmpName = string.sub("clc" .. specialization_name, 1, 14)

	-- check if a macro wtih the default name exists already and if so try another name
	local macroName, nameClash, lastChecked = tmpName
	local nextPostFix = 0
	repeat
		nameClash = false
		nextPostFix = nextPostFix + 1
		for i=1,numperchar do
			lastChecked = i
			local name = (GetMacroInfo(i + 36))
			if macroName == name then
				nameClash = true
				macroName = tmpName .. tostring(nextPostFix)
				break
			end
		end
	until (not nameClash) or (lastChecked == numperchar)
	
	-- IF we can't find a name well...
	if nameClash then
		StaticPopup_Show("CLCINFO_MONK_MACRO_CREATE_ERROR", L["You already have macros with the same name(s)."])
		return
	end

	-- get icon, using stance icons
	local iconSpellId = (module.specIdx == 1) and 115069 or 103985
	local _, _, macroIcon = GetSpellInfo(iconSpellId)
	macroIcon = string.gsub(macroIcon, "Interface\\Icons\\", "")

	-- get the prio list
	local prios = table.concat(module.tdb.priority, " ")
	local body = string.format('/clcinfo %s %s', module.cmdPrioArg, prios)
	local macroIdx = CreateMacro(macroName, macroIcon, body, 1)

	PickupMacro(macroIdx)

	clcInfo_Monk.OutputMessage(L["%s macro created."]:format(macroName))
end

local function GetCreateMacroFunc(module)
	return function () CreatePrioMacro(module) end
end

-- The following section is dedicated to new users to facilitate the setup of templates
function clcInfo_Monk.CreateTemplate(module, namePostFix)
	local defTalentIdx = 1 -- no need to do all sorts of logic to find a talent, since clcInfo now supports the All setting, which is 1
	local _, specialization_name = GetSpecializationInfo(module.specIdx)
	namePostFix = namePostFix or ""

	local t = clcInfo.templates:GetDefault()
	t.spec.talent = defTalentIdx
	t.spec.primary = module.specIdx
	t.options.udLabel = specialization_name..namePostFix

	local icons = module.CreateDefaultTemplateIcons(1)
	t.icons = icons

	local grid = clcInfo.display.grids:GetDefault()
	grid.cellsX = #icons
	grid.cellsY = 2
	grid.spacingX = 1
	grid.udLabel = "IconGrid"
	clcInfo.display.grids.unlock = true
	table.insert(t.grids, grid)

	clcInfo_Monk.OutputMessage(L["%s Template Created (/clcInfo)."]:format(specialization_name))

	return t
end

local function InsertTemplate(module, refresh, namePostFix)
	
	local t = clcInfo_Monk.CreateTemplate(module, namePostFix)

	table.insert(clcInfo.cdb.templates, t)

	if refresh then
		clcInfo:OnTemplatesUpdate()
		if clcInfo_Options then
			clcInfo_Options:UpdateTemplateList()
		end
	end
end

local function InsertDefaultTemplates()
	for _,v in ipairs(clcInfo_Monk.modules) do
		if v.templateMissing then
			InsertTemplate(v, false)
		end
	end

	clcInfo:OnTemplatesUpdate()
	if clcInfo_Options then
		clcInfo_Options:UpdateTemplateList()
	end
end

function clcInfo_Monk.TemplateExistsFor(specIdx)
	local templates = clcInfo.cdb.templates
	for _, v in ipairs(templates) do
		if v.spec and v.spec.primary == specIdx and v.spec.talent > 0 then
			return true
		end
	end
	return false
end

function clcInfo_Monk:AddTemplate(module)
	if self.TemplateExistsFor(module.specIdx) then
		local _, specialization_name = GetSpecializationInfo(module.specIdx)
		local dialog = StaticPopup_Show("CLCINFO_MONK_CONFIRM_CREATE_ADDITIONAL_TEMPLATE", specialization_name)
		if (dialog) then
			dialog.data  = module
		end
	else
		InsertTemplate(module, true)
	end
end

function clcInfo_Monk:OpenOptions()
	if not clcInfo_Options then
		local loaded, reason = LoadAddOn("clcInfo_Options")
		if( not clcInfo_Options ) then
			print("Failed to load configuration addon. Error returned: ", reason)
			return
		end
	end
	clcInfo_Options:Open()
end

-- DIALOGS
StaticPopupDialogs["CLCINFO_MONK_VERSION_INFO"] = {
	text = "Change notification clcInfo: Monk\n\nDue to 5.1 changes and other tweaks it is recommended to update the priorities.\n\nThe easiest way to do this is to click one of the recommended priority buttons on the class module page representing your specialization\n\nDo you wish to go there now?",
	button1 = YES,
	button2 = NO,
	OnAccept = function (self)
		clcInfo_Monk:OpenOptions()
		
		local curSpecIdx = GetSpecialization()
		local modName
		for i,v in ipairs(clcInfo_Monk.modules) do
			if v.specIdx == curSpecIdx then
				modName = v.moduleName
			end
		end
		local AceDialog = LibStub("AceConfigDialog-3.0")
		if modName then
			AceDialog:SelectGroup("clcInfo", "classModules", modName)
		else
			AceDialog:SelectGroup("clcInfo", "classModules")
		end
	end,
	hideOnEscape = 1,
	timeout = 0,
	preferredIndex = 3,
	whileDead = 1,
}

StaticPopupDialogs["CLCINFO_MONK_QUESTION_CREATE_MISSING_TEMPLATES"] = {
	text = L["Welcome to clcInfo: Monk\n\nPresently no templates exist for the following specs:\n\n%s\nDo you want to create those templates now?"],
	button1 = YES,
	button2 = NO,
	OnAccept = function (self)
		InsertDefaultTemplates()
	end,
	hideOnEscape = 1,
	timeout = 0,
	preferredIndex = 3,
	whileDead = 1,
}

StaticPopupDialogs["CLCINFO_MONK_CONFIRM_CREATE_ADDITIONAL_TEMPLATE"] = {
	text = L["You already have atleast one %s template, do you wish to create another?\n\nNote you will have to pick a distinguishing talent on each template."],
	button1 = YES,
	button2 = NO,
	OnAccept = function (self, module)
		InsertTemplate(module, true, L["New"])
	end,
	hideOnEscape = 1,
	timeout = 0,
	preferredIndex = 3,
	whileDead = 1,
}

StaticPopupDialogs["CLCINFO_MONK_MACRO_CREATE_ERROR"] = {
	text = L["Unable to create macro!\n\n%s"],
	button1 = OKAY,
	button2 = nil,
	OnAccept = function ()
	end,
	OnCancel = function()
    end,
	hideOnEscape = 1,
	timeout = 0,
	preferredIndex = 3,
	whileDead = 1,
}

--Create a module options screen
local function LoadActiveModuleOptions(class_module, activeTemplateName, GetDB, SetDB, GetTDB, SetTDB)
	
	local _, specialization_name = GetSpecializationInfo(class_module.specIdx)

	-- General options
    local cfgMatrix = {
        order = 2, type = "group", childGroups = "tab", name = specialization_name,
        args = {
            tabPriority = {
                order = 1, type = "group", name = ABILITIES, args = {
                    igSettings = {
                        order=1, type = "group", inline = true, name="", args = {
							activeTemplate = {
                                order = 3, type = "description", width = "double",
                                name = L["Active Template: |cff41a25d%s|r"]:format(activeTemplateName),
                            },
							createNewTemplate = {
								order = 4, type = "execute",
								name = L["Create new template"],
								desc = L["Create a new template for this specialization. The template will include a default fully configured icon setup."],
								func = GetCreateTemplateFunc(class_module)
							},
                        },
                    },
					prioRecommendedSettings = {
                        order = 2, type = "group", inline = true, name = L["Recommended Priorities"],
                        args = {} -- place holder for recommended priority buttons
                    },
                    prioSettings = {
                        order = 3, type = "group", inline = true, name = L["Priorities"],
                        args = {
							prioDesc = {
								order = 1, type = "description",
								name = L["To change prioriries, click the priority to move and then click one of the gaps between abilities to place it in its new position."],
							},
							prio = {
                            order = 4, type = "select", dialogControl = 'CLCM_Priorities', values = class_module,
                            name = "", get = GetPriorities
							},
						} 
                    },
					prioDesc = {
						order = 4, type = "description",
						name = L["The priorities can also be set from the command line or a macro using /clcinfo %s <priorities separared by spaces>."]:format(class_module.cmdPrioArg),
					},
					createMacro = {
						order = 5, type = "execute",
						name = L["Export to Macro"],
						desc = L["Export the current priority list to a macro to easily change to that priority."],
						func = GetCreateMacroFunc(class_module)
					},
                },           
            },
			tabSettings = {
				order = 3, type = "group", name = L["Settings"], args = {
					igSettings = {
                        order=1, type = "group", inline = true, name=L["General"], args = {
						    rangePerSkill = {
                                order = 2, type = "toggle", width="full",
                                name = L["Range per skill"],
								desc = L["Whether to check for range on a skill by skill basis, or simply melee range."], 
                                get = GetTDB, set = SetTDB,
							},
							tpRefresh = {
								order = 3, width="double", type = "range", min = 0, max = 15, step = 0.1, name = L["Tiger Power Refresh Threshold"],
								desc=L["The number of seconds left on the Tiger Power buff where it will be considered a priority to refresh it."], 
								get = GetTDB, set = SetTDB,
							},
							latencyAdjust = {
								order = 4, width="double", type = "range", min = 0, max = 0.5, step = 0.1, name = L["Latency Adjust"],
								desc=L["If you experience that, during a GCD, an unusable ability is briefly suggested, then adjust this up. If you never experience that, adjust it down."] , 
								get = GetTDB, set = SetTDB,
							},
							todWait = {
								order = 5, width="double", type = "range", min = 0, max = 6, step = 0.1, name = L["Touch of Death Wait Threshold"],
								desc=L["The time to wait for Touch of Death to come off cool down if it is otherwise usable (3 chi (unless glyphed) and Death Note active)."] , 
								get = GetTDB, set = SetTDB,
							},
							poolTod = {
								order = 6, width="double", type = "toggle", name = L["Pool Chi for Touch of Death"],
								desc=L["Whether to pool (save) chi for Touch of Death if it is otherwise available (off CD and Death Note active). This will bypass other chi using abilities to allow for 3 chi. If glyphed, this setting is ignored."] , 
								get = GetTDB, set = SetTDB,
							},
						},
					},
					spec = {
                        order=2, type = "group", inline = true, name=specialization_name, args = {}
					},
				},
			},
        },
    }

	-- Brewmaster specific
	if class_module.specIdx == 1 then
		local grpSettings = cfgMatrix.args.tabSettings.args.spec.args

		grpSettings.bokRefresh = {
			order = 10, width="double", type = "range", min = 0, max = 15, step = 0.1, name = L["Shuffle Refresh Threshold"],
			desc=L["The minimum number of seconds on the shuffle buff, before reducing priority to allow other chi consumers in. The effect of this is obviously dependent on priorities, the less a prio BoK is the less this will matter."], 
			get = GetTDB, set = SetTDB,
		}
		grpSettings.purStaggerUntil = {
			order = 11, width="double", type = "range", min = 0, max = 9, step = 0.1, name = L["Stagger Purification Threshold"],
			desc=L["The number of seconds left on the Stagger debuff where it is considered futile to purify it."], 
			get = GetTDB, set = SetTDB,
		}
		grpSettings.useElubAt = {
			order = 12, width="double", type = "range", min = 1, max = 15, step = 1, name = L["Elusive Brew Threshold"],
			desc=L["The number of stacks of Elusive brew at which to begin to suggest it. This also applies if it is in a separate 3rd icon."] , 
			get = GetTDB, set = SetTDB,
		}
		grpSettings.waitKs = {
			order = 13, width="double", type = "range", min = 0, max = 1, step = 0.1, name = L["Keg Smash Wait Threshold"],
			desc=L["The time to wait for Keg Smash to come off cooldown, so that it can be preferred over other energy using abilities laer in the priotity list even if they are ready to use now."] , 
			get = GetTDB, set = SetTDB,
		}
	end

	-- Windwalker specific
	if class_module.specIdx == 3 then
		local grpSettings = cfgMatrix.args.tabSettings.args.spec.args

		grpSettings.useTebAt = {
			order = 10, width="double", type = "range", min = 1, max = 10, step = 1, name = L["Tigereye Brew Threshold"],
			desc=L["The number of stacks of Tigereye brew at which to begin to suggest it. It will be suggested if it's about to time out regardless of this setting. This also applies if it is in a separate 3rd icon."] , 
			get = GetTDB, set = SetTDB,
		}
		grpSettings.useEbTh = {
			order = 11, width="double", type = "range", min = 3, max = 10, step = 0.1, name = L["Energizing Brew Threshold"],
			desc=L["The time required to reach full energy before suggesting Energizing Brew. I.e. the default of 5 will suggest using Energizing brew if it will take more than 5 seconds to reach 100 energy."] , 
			get = GetTDB, set = SetTDB,
		}
		grpSettings.poolFof = {
			order = 12, width="double", type = "toggle", name = L["Pool Chi for Fists of Fury"],
			desc=L["Whether to pool (save) chi for Fists of Fury if it is otherwise available (off CD). This will bypass other chi using abilities (apart from Touch of Death if that has higher prio) to allow for 3 chi."] , 
			get = GetTDB, set = SetTDB,
		}
		grpSettings.rskMaxDelay = {
			order = 13, width="double", type = "range", min = 0, max = 1, step = 0.1, name = L["Rising Sun Kick Max Delay"],
			desc=L["The acceptable time to delay a Rising Sun Kick due to using another chi consuming ability."] , 
			get = GetTDB, set = SetTDB,
		}
	end

	-- generate recommended priority buttons
	local prioRecommendedArgs = cfgMatrix.args.tabPriority.args.prioRecommendedSettings.args
	for k,v in pairs(class_module.recommendedPriorities) do
		prioRecommendedArgs[k] = {
			order = 1, type = "execute",
			name = k, desc = L["Click to adjust priorities and settings to the recommendations for %s play."]:format(k),
			func = v(class_module)
		}
	end

	return cfgMatrix
end

local function LoadInactiveModuleOptions(class_module, info, showCreate, showChangeSpec)
	local _, specialization_name = GetSpecializationInfo(class_module.specIdx)

	local cfgMatrix = {
        order = 2, type = "group", childGroups = "tab", name = specialization_name,
        args = {
			tabInfo = {
                order = 1, type = "group", name = L["Inactive"], 
				args = {
					prioDesc = {
						order = 1, type = "description", width="full",	name = info,
					},
				},
			},
		},
	}
	
	if(showCreate) then
	
		cfgMatrix.args.tabInfo.args.createNewTemplate = {
			order = 3, type = "execute",
			name = L["Create new template"],
			desc = L["Create a new template for this specialization. The template will include a default fully configured icon setup."],
			func = GetCreateTemplateFunc(class_module)
		}
		
	end

	if(showChangeSpec) then
		cfgMatrix.args.tabInfo.args.changeSpec = {
			order = 4, type = "execute", width="double",
			name = L["Activate your %s specialization"]:format(specialization_name),
			desc = L["Click to activate the specialization for this class module, i.e change spec."],
			func = GetChangeSpecFunc(class_module)
		}
		
	end

	return cfgMatrix
end

function clcInfo_Monk.LoadModule(class_module, GetDB, SetDB, GetTDB, SetTDB)

	if not clcInfo_Options then return end
	
	local activeTemplateSpec, activeTemplateName = clcInfo_Monk.GetActiveTemplateSpec()

	local cfgMatrix = nil
	local curSpecIdx = GetSpecialization()
	local _, curSpecName = GetSpecializationInfo(curSpecIdx)
	local _, otherSpecName = GetSpecializationInfo(class_module.specIdx)

	if activeTemplateSpec == class_module.specIdx then
		cfgMatrix = LoadActiveModuleOptions(class_module, activeTemplateName, GetDB, SetDB, GetTDB, SetTDB)
	elseif activeTemplateSpec == nil then
		-- So no active template
		if class_module.specIdx == curSpecIdx then
			cfgMatrix = LoadInactiveModuleOptions(class_module, L["There is no active template, thus options for the %s are not available.\nIf you like, you can create a new template for the %s which will then become active."]:format(curSpecName,curSpecName), true)
		else
			local offerChangeSpec = clcInfo_Monk.GetSpecGroup(class_module)
			cfgMatrix = LoadInactiveModuleOptions(class_module, L["There is no active template, thus options for the %s are not available.\nIf you like, you can create a new template for the %s, it will not become active until you change specs though."]:format(otherSpecName,otherSpecName), true, offerChangeSpec)
		end
	else
		-- So we do have an active template, but of course it only matches one of the specs, so this is for the one it didn't match
		if class_module.specIdx == curSpecIdx then
			local _, otherSpecName = GetSpecializationInfo(activeTemplateSpec)
			cfgMatrix = LoadInactiveModuleOptions(class_module, L["The active template has been forced to match the %s specialization, thus options for the %s are not available."]:format(otherSpecName, curSpecName), false)
		else
			local offerChangeSpec = clcInfo_Monk.GetSpecGroup(class_module)
			cfgMatrix = LoadInactiveModuleOptions(class_module, L["The active template is for the %s specialization (your current spec), thus options for the %s are not available."]:format(curSpecName, otherSpecName), false, offerChangeSpec)
		end
	end

	clcInfo_Options.options.args.classModules.args[class_module.moduleName] = cfgMatrix
end